import React from 'react';
import ReactDOM from 'react-dom/client';
import App from './App';
import './index.css';
import { QueryClientProvider } from "@tanstack/react-query";
import { queryClient } from "./lib/queryClient";
import { TooltipProvider } from "@/components/ui/tooltip";
import { initPerformanceMonitoring } from './lib/performance';

// Add Vite HMR types
declare global {
  interface ImportMeta {
    hot?: {
      accept: (path: string, callback: () => void) => void;
    };
  }
}

// Error Boundary Component
class ErrorBoundary extends React.Component<
  { children: React.ReactNode },
  { hasError: boolean; error: Error | null }
> {
  constructor(props: { children: React.ReactNode }) {
    super(props);
    this.state = { hasError: false, error: null };
  }

  static getDerivedStateFromError(error: Error) {
    return { hasError: true, error };
  }

  componentDidCatch(error: Error, errorInfo: React.ErrorInfo) {
    console.error('React Error Boundary caught an error:', error, errorInfo);
  }

  render() {
    if (this.state.hasError) {
      return (
        <div style={{
          padding: '20px',
          textAlign: 'center',
          maxWidth: '600px',
          margin: '40px auto',
          fontFamily: 'system-ui, -apple-system, sans-serif'
        }}>
          <h1 style={{ color: '#7c3aed', marginBottom: '16px' }}>Something went wrong</h1>
          <p style={{ color: '#4b5563', marginBottom: '24px' }}>
            We're sorry, but something went wrong. Please try refreshing the page.
          </p>
          <button
            onClick={() => window.location.reload()}
            style={{
              background: '#7c3aed',
              color: 'white',
              border: 'none',
              padding: '12px 24px',
              borderRadius: '9999px',
              fontWeight: '500',
              cursor: 'pointer'
            }}
          >
            Reload Page
          </button>
          {process.env.NODE_ENV === 'development' && (
            <pre style={{
              marginTop: '20px',
              padding: '16px',
              background: '#f3f4f6',
              borderRadius: '8px',
              overflow: 'auto',
              textAlign: 'left'
            }}>
              {this.state.error?.stack}
            </pre>
          )}
        </div>
      );
    }

    return this.props.children;
  }
}

// Initialize performance monitoring
initPerformanceMonitoring((metric) => {
  // Log performance metrics to console in development
  if (process.env.NODE_ENV === 'development') {
    console.log(`Performance metric - ${metric.name}: ${metric.value} (${metric.rating})`);
  }
  
  // In production, you might want to send these metrics to your analytics service
  if (process.env.NODE_ENV === 'production') {
    // Example: Send to analytics
    // analytics.track('performance_metric', metric);
  }
});

// Ensure the root element exists
const rootElement = document.getElementById('root');
if (!rootElement) throw new Error('Failed to find the root element');

// Create root with strict mode for development only
const root = ReactDOM.createRoot(rootElement);

const renderApp = () => {
  root.render(
    <ErrorBoundary>
      {process.env.NODE_ENV === 'development' ? (
        <React.StrictMode>
          <QueryClientProvider client={queryClient}>
            <TooltipProvider>
              <App />
            </TooltipProvider>
          </QueryClientProvider>
        </React.StrictMode>
      ) : (
        <QueryClientProvider client={queryClient}>
          <TooltipProvider>
            <App />
          </TooltipProvider>
        </QueryClientProvider>
      )}
    </ErrorBoundary>
  );
};

// Initial render with error handling
try {
  renderApp();
} catch (error) {
  console.error('Failed to render app:', error);
  if (rootElement) {
    rootElement.innerHTML = `
      <div style="
        padding: 20px;
        text-align: center;
        max-width: 600px;
        margin: 40px auto;
        font-family: system-ui, -apple-system, sans-serif;
      ">
        <h1 style="color: #7c3aed; margin-bottom: 16px;">Failed to Start Application</h1>
        <p style="color: #4b5563; margin-bottom: 24px;">
          We're sorry, but the application failed to start. Please try refreshing the page.
        </p>
        <button onclick="window.location.reload()" style="
          background: #7c3aed;
          color: white;
          border: none;
          padding: 12px 24px;
          border-radius: 9999px;
          font-weight: 500;
          cursor: pointer;
        ">
          Reload Page
        </button>
      </div>
    `;
  }
}

// Handle HMR updates
if (import.meta.hot) {
  import.meta.hot.accept('./App', () => {
    console.log('HMR: Updating App component');
    try {
      renderApp();
    } catch (error) {
      console.error('HMR update failed:', error);
    }
  });
}
